// cedar -- C++ implementation of Efficiently-updatable Double ARray trie
//  $Id: cedar.h 1222 2012-12-27 23:56:40Z ynaga $
// Copyright (c) 2013 Naoki Yoshinaga <ynaga@tkl.iis.u-tokyo.ac.jp>
#include <cstdio>
#include <cstdlib>
#include <cedar.h>

static const size_t NUM_RESULT = 1024;

int main(int argc, char **argv) {
  if (argc < 2)
    { std::fprintf (stderr, "Usage: %s trie\n", argv[0]); std::exit (1); }

  typedef cedar::da <int, -1, -2> trie_t;
  trie_t trie;

  if (trie.open (argv[1]))
    { std::fprintf (stderr, "cannot open: %s", argv[1]); std::exit (1); }

  trie_t::result_pair_type result_pair[NUM_RESULT];
  char*  line = 0;
#ifdef __APPLE__
  size_t read = 0;
  while ((line = fgetln (stdin, &read)) != NULL) {
#else
  ssize_t read = 0;
  size_t  size = 0;
  while ((read = getline (&line, &size, stdin)) != -1) {
#endif
    *(line + read - 1) = '\0';
    if (const size_t n = trie.commonPrefixSearch (line, result_pair, NUM_RESULT)) {
      std::fprintf (stderr, "%s: found, num=%ld ", line, n);
      for (size_t i = 0; i < n; ++i)
        std::fprintf (stderr, " %d:%ld", result_pair[i].value, result_pair[i].length);
      std::fprintf (stderr, "\n");
    } else {
      std::fprintf (stderr, "%s: not found\n", line);
    }
  }
  return 0;
}
